@@ -1,12 +1,12 @@
 ;;; corfu-indexed.el --- Select indexed candidates -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2022-2023 Free Software Foundation, Inc.
 
 ;; Author: Luis Henriquez-Perez <luis@luishp.xyz>, Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
 ;; Version: 0.1
-;; Package-Requires: ((emacs "27.1") (corfu "0.28"))
+;; Package-Requires: ((emacs "27.1") (corfu "0.38"))
 ;; Homepage: https://github.com/minad/corfu
 
 ;; This file is part of GNU Emacs.
@@ -27,8 +27,8 @@
 ;;; Commentary:
 
 ;; This package is a Corfu extension, which prefixes candidates with indices if
-;; enabled via `corfu-indexed-mode'. It allows you to select candidates with
-;; prefix arguments. This is designed to be a faster alternative to selecting a
+;; enabled via `corfu-indexed-mode'.  It allows you to select candidates with
+;; prefix arguments.  This is designed to be a faster alternative to selecting a
 ;; candidate with `corfu-next' and `corfu-previous'.
 
 ;;; Code:
@@ -50,17 +50,33 @@
 (defcustom corfu-indexed-start 0
   "Start of the indexing."
   :group 'corfu
-  :type 'integer)
+  :type 'natnum)
 
 (defvar corfu-indexed--commands
   '(corfu-insert corfu-complete)
   "Commands that should be indexed.")
 
-(defun corfu-indexed--affixate (cands)
-  "Advice for `corfu--affixate' which prefixes the CANDS with an index."
-  (setq cands (cdr cands))
+;;;###autoload
+(define-minor-mode corfu-indexed-mode
+  "Prefix candidates with indices."
+  :global t :group 'corfu)
+
+(cl-defmethod corfu--prepare :before (&context (corfu-indexed-mode (eql t)))
+  (when (and prefix-arg (memq this-command corfu-indexed--commands))
+    (let ((index (+ corfu--scroll
+                    (- (prefix-numeric-value prefix-arg)
+                       corfu-indexed-start))))
+      (if (and (>= index 0)
+               (< index corfu--total)
+               (< index (+ corfu--scroll corfu-count)))
+          (setq corfu--index index)
+        (message "Out of range")
+        (setq this-command #'ignore)))))
+
+(cl-defmethod corfu--affixate :around (cands &context (corfu-indexed-mode (eql t)))
+  (setq cands (cdr (cl-call-next-method cands)))
   (let* ((space #(" " 0 1 (face (:height 0.5 :inherit corfu-indexed))))
-         (width (if (> (+ corfu-indexed-start (length cands)) 10) 2 1))
+         (width (if (length> cands (- 10 corfu-indexed-start)) 2 1))
          (fmt (concat space
                       (propertize (format "%%%ds" width)
                                   'face 'corfu-indexed)
@@ -77,32 +93,5 @@
              (cadr cand))))
     (cons t cands)))
 
-(defun corfu-indexed--handle-prefix (orig &rest args)
-  "Handle prefix argument before calling ORIG function with ARGS."
-  (if (and current-prefix-arg (called-interactively-p t))
-      (let ((corfu--index (+ corfu--scroll
-                             (- (prefix-numeric-value current-prefix-arg)
-                                corfu-indexed-start))))
-        (if (or (< corfu--index 0)
-                (>= corfu--index corfu--total)
-                (>= corfu--index (+ corfu--scroll corfu-count)))
-            (message "Out of range")
-          (funcall orig)))
-    (apply orig args)))
-
-;;;###autoload
-(define-minor-mode corfu-indexed-mode
-  "Prefix candidates with indices."
-  :global t :group 'corfu
-  (cond
-   (corfu-indexed-mode
-    (advice-add #'corfu--affixate :filter-return #'corfu-indexed--affixate)
-    (dolist (cmd corfu-indexed--commands)
-      (advice-add cmd :around #'corfu-indexed--handle-prefix)))
-   (t
-    (advice-remove #'corfu--affixate #'corfu-indexed--affixate)
-    (dolist (cmd corfu-indexed--commands)
-      (advice-remove cmd #'corfu-indexed--handle-prefix)))))
-
 (provide 'corfu-indexed)
 ;;; corfu-indexed.el ends here
